/*========================================================================================
(c) Dmitriy N. Sakharov
TableFit v2.5

ОПИСАНИЕ
Скрипт за счет пропорционального увеличения отступов между строками таблицы (в заданных пределах)
растягивает ее по вертикали до нижней границы фрейма на каждой полосе.
Можно исключать обработку начала или конца таблицы.
Максимально возможные значения отступов в строке устанавливаются заранее.
При превышении максимального значения отступа выводится предупреждение.

ИНСТРУКЦИЯ
Перед запуском скрипта необходимо поставить курсор в таблицу, которую нужно обрабатывать.

ОГРАНИЧЕНИЯ
Обработка проводиться не будет, если:
1) в таблицах есть Footer Rows;
2) фрейм, в котором находится таблица, повернут на угол отличный от 0 или 90 град.
При необходимости перечисленные ограничения можно исключить в следующей версии скрипта.

Вопросы и замечания по работе скрипта направляйте на sakhar@mail.ru
=========================================================================================*/


with (app) {
	// Проверки	
	if(app.documents.length==0){
		alert("Нет открытых документов!");
		exit();
	}
	if(selection.length==0){
		alert("Поставьте курсор в таблицу и попробуйте еще раз!");
		exit();
	}
	switch (app.selection[0].constructor.name){
    	case "InsertionPoint": var myTable = app.selection[0].parent.parent; break;
    	case "Text": var myTable = app.selection[0].parent.parent; break;
    	case "Word": var myTable = app.selection[0].parent.parent; break;
    	case "Character": var myTable = app.selection[0].parent.parent; break;
    	case "Paragraph": var myTable = app.selection[0].parent.parent; break;
    	case "Cell": var myTable = app.selection[0].parent; break;
    	case "Table": var myTable = app.selection[0]; break;
    	default: alert("Поставьте курсор в таблицу и попробуйте еще раз!"); exit();
	}	
	if(myTable.constructor.name != "Table"){
			alert("Поставьте курсор в таблицу и попробуйте еще раз!");
			exit();
	}
	if(myTable.footerRowCount != 0){
			alert("В таблице присутствует footerRow.\nК сожалению, обработка невозможна.");
			exit();
	}
	// Переменные
	var myRows = myTable.rows;
    var myLim = myRows.length;		// Количество строк в таблице
    var j;							// Номер строки в таблице
	var myDelta;					// Приращение 
    var myPageErr =0 ;				// Страница, на которой таблица осталась недотянутой
    var myTopLim = 1.400;			// Максимальный отступ строки сверху
    var myBottomLim = 1.400;		// Максимальный отступ строки снизу
    var myTableBegin = 0;			// Начало таблицы
    var myTableEnd = 0;    			// Конец таблицы	
    var myFlag;
    var myFix = 0;
    var myTopW = 0;
	var myBottomW = 0;
    var m = 0;
    var myAlert = "\nК сожалению, обработка невозможна.";
    var check = false;
    // Устанавливаем единицы измерения в миллиметрах
	var userHoriz = app.activeDocument.viewPreferences.horizontalMeasurementUnits;
	var userVert =  app.activeDocument.viewPreferences.verticalMeasurementUnits;
	app.activeDocument.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.millimeters;
	app.activeDocument.viewPreferences.verticalMeasurementUnits = MeasurementUnits.millimeters;
    // Создаем диалог
	var myDialog = dialogs.add({name:"TableFit v2.5"});
	with (myDialog.dialogColumns.add()) {
		with (dialogRows.add().borderPanels.add().dialogColumns.add()) {
			with (dialogRows.add()) {				
				staticTexts.add({staticLabel:"Обрабатывать: "});
				with (dialogColumns.add()) {				
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"начало таблицы",minWidth:95});
						var myDialogTableBegin = checkboxControls.add({});
						myDialogTableBegin.checkedState = true;
					}
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"конец таблицы",minWidth:95});
						var myDialogTableEnd = checkboxControls.add({});
						myDialogTableEnd.checkedState = true;
					}
				}
			}
			with (dialogRows.add()) {							
				staticTexts.add({staticLabel:"Допустимые отступы (мм): "});
				with (dialogColumns.add()) {
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"сверху",minWidth:40});
						var myDialogTopLim = realEditboxes.add({editValue:myTopLim});
					}
					with (dialogRows.add()) {
						staticTexts.add({staticLabel:"снизу",minWidth:40});
						var myDialogBottomLim = realEditboxes.add({editValue:myBottomLim});
					}
				}
			}
			with (dialogRows.add()) {							
				staticTexts.add({staticLabel:"Выводить предупреждения, отчет "});
				var myDialogInfo = checkboxControls.add({});
				myDialogInfo.checkedState = false;
			}
		}
		with (dialogRows.add()) {			
			staticTexts.add({staticLabel:"(с) Dmitriy N. Sakharov    sakhar@mail.ru"});
		}
	}
	// Показываем
	var myRes = myDialog.show();
	if (!myRes) {
		myDialog.destroy();
		exit();
	}
	var mytime0 = new Date(); // Запуск таймера	
	if (!myDialogTableBegin.checkedState) {myTableBegin = 1;}
	if (!myDialogTableEnd.checkedState) {myTableEnd = 1;}
	myTopLim = myDialogTopLim.editValue;
	myBottomLim = myDialogBottomLim.editValue;
	// Проверка введенных данных
	if (myTopLim < 0 || myBottomLim < 0) {
		alert("Заданный отступ невозможен!");
		exit();
	}		 
    var myStartRows = new Array();    		// Массив номеров первых строк таблицы в каждом фрейме
    myStartRows[0] = myTable.headerRowCount;
    var i = 1;								// Счетчик фреймов
    var myPrevTF = myRows[myTable.headerRowCount].cells[0].texts[0].parentTextFrames[0];
	// Заполнение массива первых строк
	for (j = myTable.headerRowCount; j < myLim; j++) {
        myCurTF = myRows[j].cells[0].texts[0].parentTextFrames[0];
        if (myCurTF != myPrevTF) {
        	myStartRows[i] = j;
        	i++;
        }
		myPrevTF = myCurTF;
    }
    myStartRows[i] = myLim;
    var myLength = myStartRows.length; // Количество фреймов, которые занимает таблица
    // Растягивание таблицы до границы фрейма на каждой полосе
    for (q = myTableBegin; q < (myLength - 1 - myTableEnd); q++) {
    	myFlag = 0;
    	myCurLastRow = myRows[(myStartRows[q+1]-1)]; // Последняя строка в текущем фрейме
    	myDelta = myDeltaFunction();
    	// Обработка всех строк таблицы, которые находятся во фрейме
    	for (x = myStartRows[q]; x <= (myStartRows[q+1] - 1); x++) {	    				
			// Проверка высоты строки
			if (!myRows[x].autoGrow || myRows[x].minimumHeight == myRows[x].height) {
				if (myFix == 0 && myDialogInfo.checkedState) {
					var myState = confirm("Установлена фиксированная (или минимальная) высота строки!\nИзменять в подобных строках высоту вместо отступов?");
					if (!myState) {
						alert("В таком случае дальнейшая обработка этой таблицы невозможна!");
						exit();
					}
					myFix = 1;
				}
				myRows[x].height += (myDelta*2);
			}
			else {
				// Отступ сверху 
				if ((myRows[x].topInset + myDelta) < myTopLim) {
					myRows[x].topInset += myDelta;
				}
				else {
					myRows[x].topInset = myTopLim;
					myFlag = 1;
				}
				// Отступ снизу
				if ((myRows[x].bottomInset + myDelta) < myBottomLim) {					
					myRows[x].bottomInset += myDelta;
				}
				else {
					myRows[x].bottomInset = myBottomLim;
					myFlag = 1;
				}
			}			
		}
		// Проверка: Дотянута ли таблица до края
		if (myFlag == 1 && check == false) {
			myPageErr = myCurLastRow.cells[0].texts[0].parentTextFrames[0].parent.name;
			check = myAtention("На странице " + myPageErr + " таблица до края фрейма не дотянута,", "т. к. превышены значения максимальных отступов!");
		}
		// Проверка: Не уползла ли строка
		if (myRows[myStartRows[q]].cells[0].texts[0].parentTextFrames[0] != myRows[(myStartRows[q+1] - 1)].cells[0].texts[0].parentTextFrames[0]) {
			alert("Произошла неизвестная ошибка.\nСейчас она будет исправлена,\nно пожалуйста, сообщите автору об ошибке!");
			myRescue();
		}
    }
    // Восстанавливаем единицы измерения
    app.activeDocument.viewPreferences.horizontalMeasurementUnits = userHoriz;
	app.activeDocument.viewPreferences.verticalMeasurementUnits = userVert;	
	if (myDialogInfo.checkedState) {	
		// Время работы скрипта
		var mytime1 = new Date(); 
		var mytimediff = Math.floor((mytime1.getTime() - mytime0.getTime()) / 1000);
	    var mymin = Math.floor(mytimediff / 60); 
		var mysec = Number(mytimediff % 60); 
		var myEndSave = 1;  
		alert("Обработка завершена!\n" + "Время работы: " + mymin + " мин " + mysec + " с" + "\n\n(с) Dmitriy N. Sakharov \n sakhar@mail.ru");	 
	}
}

// Функция расчета приращения
function myDeltaFunction()
{	
	var myCurTextFrame = myCurLastRow.cells[0].texts[0].parentTextFrames[0];
	// Проверка угла поворота фрейма
	switch (myCurTextFrame.rotationAngle) {
		case 0: m = 2; break;
		case 90: m = 3; break;
		default: alert("Недопустимый угол поворота фрейма!" + myAlert); exit();
	}
	// Целевая координата	
	var myCurTextFrameJust = myCurTextFrame.textFramePreferences.verticalJustification;
	myCurTextFrame.textFramePreferences.verticalJustification = 1651471469;		
	if (m == 2) {
		myDelta = Math.abs(myCurLastRow.cells[0].texts[0].insertionPoints[0].baseline);
	}
	else {
		myDelta = Math.abs(myCurLastRow.cells[0].texts[0].insertionPoints[0].horizontalOffset);
	}
	myCurTextFrame.textFramePreferences.verticalJustification = myCurTextFrameJust;
	// Вычитаем фактическую координату
	if (m == 2) {
		myDelta -= Math.abs(myCurLastRow.cells[0].texts[0].insertionPoints[0].baseline);
	}
	else {
		myDelta -= Math.abs(myCurLastRow.cells[0].texts[0].insertionPoints[0].horizontalOffset);
	}
	myDelta = Math.abs(myDelta);
	// Делим на количество строк во фрейме и на 2 (верх и низ)
	myDelta = myDelta / ((myStartRows[q+1]) - (myStartRows[q])) / 2;
	return myDelta
}

// Функция предупреждения о превышении предела
function myAtention(mytext1, mytext2)
{
	// Создаем диалог
    var myDialog2 = app.dialogs.add({name:"TableFit v2.5"});
    with (myDialog2.dialogColumns.add()) {
    	with (dialogRows.add().borderPanels.add().dialogColumns.add()) {
    		with (dialogRows.add()) {staticTexts.add({staticLabel:mytext1});}
    		with (dialogRows.add()) {staticTexts.add({staticLabel:mytext2});}
    	}
		with (dialogRows.add()) {
   			var showStoped = checkboxControls.add({staticLabel:"Больше не показывать"});
   		}
       	with (dialogRows.add()) {
			staticTexts.add({staticLabel:"(с) Dmitriy N. Sakharov    sakhar@mail.ru"});
		}
    }
	var myResult = myDialog2.show();
	if (!myResult) {
		myDialog.destroy();
		exit();
	}
	return showStoped.checkedState 
}

// Функция аварийного исправления
function myRescue()
{
	var myFlag1 = 0;
   	do {
   		for (x = myStartRows[q]; x <= (myStartRows[q+1] - 1); x++) {			
			myRows[x].topInset = myRows[x].topInset - 0.01;
			myRows[x].bottomInset = myRows[x].bottomInset - 0.01;
			if (myRows[myStartRows[q]].cells[0].texts[0].parentTextFrames[0] == myRows[(myStartRows[q+1] - 1)].cells[0].texts[0].parentTextFrames[0]) {				
				myFlag1 = 1;
				break;
			}
		}
   	}while (myFlag1 == 0)
}